@@ -280,6 +280,58 @@ pdf_document_load (EvDocument   *document,
 	return TRUE;
 }
 
+#ifdef HAVE_POPPLER_DOCUMENT_NEW_FROM_STREAM
+static gboolean
+pdf_document_load_stream (EvDocument          *document,
+                          GInputStream        *stream,
+                          EvDocumentLoadFlags  flags,
+                          GCancellable        *cancellable,
+                          GError             **error)
+{
+        GError *err = NULL;
+        PdfDocument *pdf_document = PDF_DOCUMENT (document);
+
+        pdf_document->document =
+                poppler_document_new_from_stream (stream, -1,
+                                                  pdf_document->password,
+                                                  cancellable,
+                                                  &err);
+
+        if (pdf_document->document == NULL) {
+                convert_error (err, error);
+                return FALSE;
+        }
+
+        return TRUE;
+}
+#endif
+
+#ifdef HAVE_POPPLER_DOCUMENT_NEW_FROM_GFILE
+static gboolean
+pdf_document_load_gfile (EvDocument          *document,
+                         GFile               *file,
+                         EvDocumentLoadFlags  flags,
+                         GCancellable        *cancellable,
+                         GError             **error)
+{
+        GError *err = NULL;
+        PdfDocument *pdf_document = PDF_DOCUMENT (document);
+
+        pdf_document->document =
+                poppler_document_new_from_gfile (file,
+                                                 pdf_document->password,
+                                                 cancellable,
+                                                 &err);
+
+        if (pdf_document->document == NULL) {
+                convert_error (err, error);
+                return FALSE;
+        }
+
+        return TRUE;
+}
+#endif
+
 static int
 pdf_document_get_n_pages (EvDocument *document)
 {
@@ -887,6 +939,12 @@ pdf_document_class_init (PdfDocumentClass *klass)
 
 	ev_document_class->save = pdf_document_save;
 	ev_document_class->load = pdf_document_load;
+#ifdef HAVE_POPPLER_DOCUMENT_NEW_FROM_STREAM
+        ev_document_class->load_stream = pdf_document_load_stream;
+#endif
+#ifdef HAVE_POPPLER_DOCUMENT_NEW_FROM_GFILE
+        ev_document_class->load_gfile = pdf_document_load_gfile;
+#endif
 	ev_document_class->get_n_pages = pdf_document_get_n_pages;
 	ev_document_class->get_page = pdf_document_get_page;
 	ev_document_class->get_page_size = pdf_document_get_page_size;
@@ -1006,6 +1064,11 @@ pdf_document_fonts_fill_model (EvDocumentFonts *document_fonts,
 		const char *name;
 		const char *type;
 		const char *embedded;
+		const gchar *substitute;
+		const gchar *substitute_text;
+		const gchar *filename;
+		const gchar *encoding;
+		const gchar *encoding_text;
 		char *details;
 		
 		name = poppler_fonts_iter_get_name (iter);
@@ -1014,6 +1077,10 @@ pdf_document_fonts_fill_model (EvDocumentFonts *document_fonts,
 			name = _("No name");
 		}
 
+		encoding = poppler_fonts_iter_get_encoding (iter);
+		if (!encoding)
+			encoding = _("None");
+
 		type = font_type_to_string (
 			poppler_fonts_iter_get_font_type (iter));
 
@@ -1026,7 +1093,17 @@ pdf_document_fonts_fill_model (EvDocumentFonts *document_fonts,
 			embedded = _("Not embedded");
 		}
 
-		details = g_markup_printf_escaped ("%s\n%s", type, embedded);
+		substitute = poppler_fonts_iter_get_substitute_name (iter);
+		filename = poppler_fonts_iter_get_file_name (iter);
+		encoding_text = _("Encoding");
+		substitute_text = _("substituting with");
+
+		if (substitute && filename)
+			details = g_markup_printf_escaped ("%s\n%s: %s\n%s, %s <b>%s</b>\n(%s)",
+							   type, encoding_text, encoding, embedded,
+							   substitute_text, substitute, filename);
+		else
+			details = g_markup_printf_escaped ("%s\n%s: %s\n%s", type, encoding_text, encoding, embedded);
 
 		gtk_list_store_append (GTK_LIST_STORE (model), &list_iter);
 		gtk_list_store_set (GTK_LIST_STORE (model), &list_iter,
