@@ -78,6 +78,9 @@ gboolean         ev_document_model_get_continuous    (EvDocumentModel *model);
 void             ev_document_model_set_dual_page     (EvDocumentModel *model,
 						      gboolean         dual_page);
 gboolean         ev_document_model_get_dual_page     (EvDocumentModel *model);
+void             ev_document_model_set_dual_page_odd_pages_left (EvDocumentModel *model,
+								 gboolean         odd_left);
+gboolean         ev_document_model_get_dual_page_odd_pages_left (EvDocumentModel *model);
 void             ev_document_model_set_fullscreen    (EvDocumentModel *model,
 						      gboolean         fullscreen);
 gboolean         ev_document_model_get_fullscreen    (EvDocumentModel *model);
